/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : feed.c
 *  Author    : I.Lea
 *  Created   : 31-08-91
 *  Updated   : 03-09-91
 *  Release   : 1.0
 *  Notes     : provides same interface to mail,pipe,print and save commands
 *  Copyright : (c) Copyright 1991 by Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

extern char *glob_group;			/* Group name */
extern char note_h_subj[LEN+1];		/* Subject:	*/
extern char note_h_from[LEN+1];		/* From:	*/
extern FILE *note_fp;				/* the body of the current article */
extern int note_end;				/* end of article ? */
extern int note_page;				/* what page we're on */
extern long note_mark[MAX_PAGES];	/* ftells on beginnings of pages */

char default_regex_pattern[LEN+1];


int feed_articles (function, level, prompt, respnum, group_path)
	int function;
	int level;
	char *prompt;
	int respnum;
	char *group_path;
{
	char address[LEN+1];
	char command[LEN+1];
	char file[LEN+1], *p;
	char mailbox[LEN+1];
	char pattern[LEN+1];
	char ch = 'a', ch_default = 'a';
	FILE *fp;
	int b, i, j, count = 1;
	int error = FALSE;
	int is_mailbox = FALSE;
	int orig_note_end;
	int orig_note_page;
	int ret1, ret2;
	int saved_ok = FALSE;
	int redraw_screen = FALSE;
	
	if (level == PAGE_LEVEL) {
		orig_note_end = note_end;
		orig_note_page = note_page;
	}

	b = which_base (respnum);

	if (num_of_tagged_files) {
		ch_default = 'T';
	}
	if (! num_of_tagged_files && nresp (b)) {
		ch_default = 't';
	}
	do {
		sprintf (msg, "%s%s%c", prompt, txt_art_thread_regex_tag, ch_default);
		wait_message (msg);
		MoveCursor (LINES, strlen (msg)-1);
		if ((ch = ReadCh ()) == CR)
			ch = ch_default;
	} while (ch != 'a' && ch != 't' && ch != 'T' && ch != 'r');

	if (ch == 'r') {
		sprintf (msg, txt_feed_pattern, default_regex_pattern);
		if (! parse_string (msg, pattern)) {
			clear_message ();
			return FALSE;
		}	
		if (strlen (pattern)) {
			my_strncpy (default_regex_pattern, pattern, LEN);
		} else {
			if (default_regex_pattern[0]) {
				my_strncpy (pattern, default_regex_pattern, LEN);
			} else {
				info_message (txt_no_match);
				return FALSE;
			}
		}
	}

	switch (function) {
		case FEED_MAIL:
			if (ch != 'a') {
				if (! parse_string (txt_mail_art_to, address)) {
					clear_message ();
					return FALSE;
				}	
				if (address[0] == '\0') {
					info_message (txt_no_mail_address);
					return FALSE;
				}
			}
			break;
		case FEED_PIPE:
			if (! parse_string (txt_pipe_to_command, command)) {
				clear_message ();
				return FALSE;
			}
			if (command[0] == '\0') {
				info_message (txt_no_command);
				return FALSE;
			}
			if ((fp = popen (command, "w")) == NULL) {
				error_message (txt_command_failed_s, command);
				return FALSE;
			}
			Raw (FALSE);
			break;
		case FEED_PRINT:	
			if (default_printer) {
				sprintf (command, "%s -P%s > /dev/null 2>&1",
					printer, get_val ("PRINTER","ps0"));
			} else {
				sprintf (command, "%s > /dev/null 2>&1", printer);
			}
			if ((fp = popen (command, "w")) == NULL) {
				error_message (txt_command_failed_s, command);
				return FALSE;
			}
			break;
		case FEED_SAVE:		/* ask user for filename */
			free_save_array ();
			if ((save_archive_name == FALSE || arts[respnum].archive == (char *) 0)) {
				if (! parse_string (txt_save_filename, file)) {
					clear_message ();
					return FALSE;
				}
				if (file[0] == '\0') {
					info_message (txt_no_filename);
					return FALSE;
				}
				for (p = file; *p && (*p == ' ' || *p == '\t'); p++) {
					;
				}
				if (! *p) {
					info_message (txt_no_filename);
					return FALSE;
				}
				if (is_mailbox = create_path (file)) {
					if ((int) strlen (file) > 1) {
						my_strncpy (mailbox, file+1, LEN);		
					} else {
						my_strncpy (mailbox, glob_group, LEN);
						/*
						 *  convert 1st letter to uppercase
						 */
						if (mailbox[0] >= 'a' && mailbox[0] <= 'z') {
							mailbox[0] = mailbox[0] - 32;
						}
					}
					my_strncpy (file, mailbox, LEN);
				}
			}
			clear_message ();
			break;
	}
	
	switch (ch) {
		case 'a':		/* article */
			if (level == GROUP_LEVEL) {
				open_note (arts[respnum].artnum, group_path);	
			}
			switch (function) {
				case FEED_MAIL:
					redraw_screen = mail_to_someone (TRUE, "");
					break;
				case FEED_PIPE:
					fseek (note_fp, 0L, 0);
					copy_fp (note_fp, fp, "");
					break;
				case FEED_PRINT:
					wait_message (txt_printing);
					if (print_header) {
						fseek(note_fp, 0L, 0);
					} else {
						fprintf (fp, "From: %s\n", note_h_from);
						fprintf (fp, "Subject: %s\n\n", note_h_subj);
						fseek (note_fp, note_mark[0], 0);
					}
					copy_fp(note_fp, fp, "");
					pclose (fp);		
					break;
				case FEED_SAVE:
					add_to_save_list (0, &arts[respnum], is_mailbox, file);
					saved_ok = save_art_to_file (respnum, 0, FALSE, NULL);
					break;
			}
			if (level == GROUP_LEVEL) {
				note_cleanup ();
			}
			break;
			
		case 't': 		/* thread */
			for (i = base[b]; i >= 0; i = arts[i].thread) {
				if (function == FEED_PRINT) {
					if ((fp = popen (command, "w")) == NULL) {
						error_message (txt_command_failed_s, command);
						return FALSE;
					}
				}
				if (level == PAGE_LEVEL) {
					note_cleanup ();
				}
				open_note (arts[i].artnum, group_path);	
				switch (function) {
					case FEED_MAIL:
						mail_to_someone (FALSE, address);
						break;
					case FEED_PIPE:
						fseek (note_fp, 0L, 0);
						copy_fp (note_fp, fp, "");
						break;
					case FEED_PRINT:
						sprintf (msg, "%s%d", txt_printing, count++);
						wait_message (msg);
						if (print_header) {
							fseek(note_fp, 0L, 0);
						} else {
							fprintf (fp, "From: %s\n", note_h_from);
							fprintf (fp, "Subject: %s\n\n", note_h_subj);
							fseek (note_fp, note_mark[0], 0);
						}
						copy_fp(note_fp, fp, "");
						pclose (fp);
						break;
					case FEED_SAVE:
						add_to_save_list (i, &arts[i], is_mailbox, file);
						break;
				}
			}
			if (function == FEED_SAVE) {
				sort_save_list ();
				saved_ok = save_thread_to_file (respnum, is_mailbox, group_path);
			}
			break;

		case 'T': 		/* tagged articles */
			for (i=1 ; i <= num_of_tagged_files ; i++) {
				for (j=0 ; j < top ; j++) {
					if (arts[j].tagged && arts[j].tagged == i) { 
						if (function == FEED_PRINT) {
							if ((fp = popen (command, "w")) == NULL) {
								error_message (txt_command_failed_s, command);
								return FALSE;
							}
						}
						if (level == PAGE_LEVEL) {
							note_cleanup ();
						}
						open_note (arts[j].artnum, group_path);	
						switch (function) {
							case FEED_MAIL:
								mail_to_someone (FALSE, address);
								break;
							case FEED_PIPE:
								fseek (note_fp, 0L, 0);
								copy_fp (note_fp, fp, "");
								break;
							case FEED_PRINT:
								sprintf (msg, "%s%d", txt_printing, count++);
								wait_message (msg);
								if (print_header) {
									fseek(note_fp, 0L, 0);
								} else {
									fprintf (fp, "From: %s\n", note_h_from);
									fprintf (fp, "Subject: %s\n\n", note_h_subj);
									fseek (note_fp, note_mark[0], 0);
								}
								copy_fp(note_fp, fp, "");
								pclose (fp);
								break;
							case FEED_SAVE:
								add_to_save_list (j, &arts[j], is_mailbox, file);
								break;
						}
					}
				}
			}
			if (function == FEED_SAVE) {				
				saved_ok = save_regex_arts (FALSE, pattern, is_mailbox, group_path);
			}
			break;

		case 'r': 		/* regex pattern matched articles */
			for (i=0 ; i < top ; i++) {
#ifdef DONT_USE_REGEX 
				if (str_str (arts[i].subject, pattern) != 0) {
#else		
				if (wildmat (arts[i].subject, pattern)) {
#endif		
					if (function == FEED_PRINT) {
						if ((fp = popen (command, "w")) == NULL) {
							error_message (txt_command_failed_s, command);
							return FALSE;
						}
					}
					if (level == PAGE_LEVEL) {
						note_cleanup ();
					}
					open_note (arts[i].artnum, group_path);	
					switch (function) {
						case FEED_MAIL:
							mail_to_someone (FALSE, address);
							break;
						case FEED_PIPE:
							fseek (note_fp, 0L, 0);
							copy_fp (note_fp, fp, "");
							break;
						case FEED_PRINT:
							sprintf (msg, "%s%d", txt_printing, count++);
							wait_message (msg);	
							if (print_header) {
								fseek(note_fp, 0L, 0);
							} else {
								fprintf (fp, "From: %s\n", note_h_from);
								fprintf (fp, "Subject: %s\n\n", note_h_subj);
								fseek (note_fp, note_mark[0], 0);
							}
							copy_fp(note_fp, fp, "");
							pclose (fp);
							break;
						case FEED_SAVE:
							add_to_save_list (i, &arts[i], is_mailbox, file);
							break;
					}
				}
			}
			if (function == FEED_SAVE) {				
				sort_save_list ();
				saved_ok = save_regex_arts (TRUE, pattern, is_mailbox, group_path);
			}
			break;
	}

	switch (function) {
		case FEED_PIPE:
			pclose (fp);		
			Raw (TRUE);
			continue_prompt ();
			redraw_screen = TRUE;
			break;
		case FEED_PRINT:	
			info_message (txt_printed);
			redraw_screen = mail_check ();	/* in case of sending to oneself */
			break;
		case FEED_SAVE:
			ret1 = (mark_saved_read ? TRUE : FALSE);
			if (saved_ok) {
				ret2 = post_process_files ();
			}
			if (ret1 || ret2) {
				redraw_screen = TRUE;
			}
			free_save_array ();
			break;
	}

	if (level == PAGE_LEVEL) {
		if (ch != 'a') {
			open_note (arts[respnum].artnum, group_path);
		}
		note_end = orig_note_end;
		note_page = orig_note_page;
		fseek (note_fp, note_mark[note_page], 0);
		if (redraw_screen) {
			if (note_page == 0) {
				show_note_page (respnum, glob_group);
			} else {
				redraw_page (respnum, glob_group);
			}
		}
	} else {
		if (redraw_screen) {
			show_group_page (glob_group);
		}
	}
}
