/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : main.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 30-08-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

char *version = "v1.0";
char cvers[LEN+1];
char nntp_server[LEN+1];

int NOTESLINES;						/* set in set_win_size () */
int RIGHT_POS;						/* set in set_win_size () */
int MORE_POS;						/* set in set_win_size () */
int max_subj = 0;
int max_from = 0;
int max_active;
int group_hash[TABLE_SIZE];			/* group name --> active[] */
int *my_group;						/* .newsrc --> active[] */
int *unread;						/* highest art read in group */
int num_active;						/* one past top of active */
int local_top;						/* one past top of my_group */
int catchup = FALSE;				/* mark all arts read in all subscribed groups */
int update = FALSE;					/* update index files only mode */
int verbose = FALSE;				/* update index files only mode */
int start_line_offset;				/* used by invoke_editor for line no. */
int read_news_via_nntp = FALSE;		/* read news locally or via NNTP */
int max_art;
int top = 0;
int top_base;
int tin_uid;
int tin_gid;
int real_uid;
int real_gid;
int local_index;				/* do private indexing? */

long *base;

struct group_ent *active;			/* active file */
struct header *arts;


void main (argc, argv)
	int argc;	
	char *argv[];
{
	extern int optind, opterr;
	extern char *optarg;
	extern char author_search_string[LEN+1];
	extern char group_search_string[LEN+1];
	extern char subject_search_string[LEN+1];
	int c, i;
	struct stat st;
	
	debug = FALSE;

	set_signal_handlers ();

	strcpy (progname, argv[0]);

	nntp_server[0] = '\0';
	group_search_string[0] = '\0';
	author_search_string[0] = '\0';
	subject_search_string[0] = '\0';

	hash_init ();
	for (i = 0; i < TABLE_SIZE; i++) {
		group_hash[i] = -1;
	}

	tin_uid = geteuid ();
	tin_gid = getegid ();
	real_uid = getuid ();
	real_gid = getgid ();

	init_selfinfo ();	/* set up char *'s: homedir, newsrc, etc. */
	init_alloc ();		/* allocate initial array sizes */

	if (tin_uid == real_uid) {	/* run out of someone's account */
		local_index = TRUE;		/* index in their home directory */
	} else {			/* we're setuid, index in /usr/spool/news */
		local_index = FALSE;
	}

	while ((c = getopt (argc, argv, "cd:Df:hm:M:p:rS:uv")) != EOF) {
		switch(c) {
			case 'c':
				catchup = TRUE;
				update = TRUE;
				break;
				
			case 'd':
				my_strncpy (savedir, optarg, LEN);
				set_real_uid_gid ();
				if (stat (savedir, &st) == -1) {
					mkdir (savedir, 0755);
				}
				set_tin_uid_gid ();
				break;

			case 'D':		/* debug mode */
				debug = TRUE;
				break;

			case 'f':
				my_strncpy (newsrc, optarg, LEN);
				break;

			case 'm':
				my_strncpy (mailer, optarg, LEN);
				break;

			case 'M':
				my_strncpy (maildir, optarg, LEN);
				set_real_uid_gid ();
				if (stat (maildir, &st) == -1) {
					mkdir (maildir, 0755);
				}
				set_tin_uid_gid ();
				break;

			case 'p':
				my_strncpy (printer, optarg, LEN);
				default_printer = FALSE;
				break;

			case 'r':	/* read news remotely from default NNTP server */
#ifdef USE_NNTP			
				read_news_via_nntp = TRUE;
#else
				printf ("Option not enabled. Recompile %s with -DUSE_NNTP.\n", progname);
				exit (1);
#endif
				break;

			case 'S':
				my_strncpy (spooldir, optarg, LEN);
				break;

			case 'u':
				update = TRUE;
				break;

			case 'v':
				verbose = TRUE;
				update = TRUE;
				break;

			case 'h':
			case '?':
			default:
				usage (progname);
				exit (1);
		}
	}

	sprintf (page_header, "%s %s PL%d%s", progname, version, PATCHLEVEL, is_remote ()); 	
	sprintf (cvers, "%s (c) Copyright 1991 Iain Lea.", page_header);

	if (! update) {
		printf ("%s\n", cvers);
		fflush (stdout);
	}

	nntp_startup ();	/* connect server if we are using nntp */
	read_active ();		/* load the active file into active[] */

	if (optind < argc) {
		while (optind < argc) {
			if (add_group (argv[optind], TRUE) < 0) {
				fprintf (stderr, txt_not_in_active_file, argv[optind]);
				fprintf (stderr, "\n");
			}
			optind++;
		}
	} else {
		read_newsrc (TRUE);
	}

	if (kill_articles) {			/* read in users kill file */
		read_kill_file ();
	}
	
	if (update) {				/* index file updater only */
		COLS = DEFAULT_COLS;	/* set because curses has not started */ 
		max_subj = (COLS/2) - 2;
		max_from = (COLS-max_subj) - 17;
		do_update ();
		exit (0);
	}

	if (InitScreen () == FALSE) {
		fprintf (stderr,txt_screen_init_failed);
		exit (1);
	}

	ScreenSize (&LINES, &COLS);		/* screen size from termcap entry */
	Raw (TRUE);

	set_win_size (&LINES, &COLS);	/* screen size doing ioctl() call */
	
	max_subj = (COLS/2) - 2;
	max_from = (COLS-max_subj) - 17;

	init_screen_array (TRUE);		/* allocate screen array */
	
	mail_setup ();		/* record mailbox size for "you have mail" */

#ifndef USE_CLEARSCREEN
	ClearScreen();
#endif

	selection_index ();

	tin_done (0);
}

/*
 * usage
 */

void usage (progname)
	char *progname;
{
    fprintf(stderr, "%s %s PL%d - threaded usenet newsreader. Copyright 1991 Iain Lea.\n\n", progname, version, PATCHLEVEL);
    fprintf(stderr, "Usage: %s [options] [newsgroups]\n",progname);
    fprintf(stderr, "  -c       mark all articles as read in subscribed newsgroups\n");
    fprintf(stderr, "  -d dir   save articles to directory [default=%s]\n",savedir);
    fprintf(stderr, "  -f file  subscribed newsgroups file [default=%s/.newsrc]\n", homedir);
    fprintf(stderr, "  -h       help\n");
    fprintf(stderr, "  -m file  mail program [default=%s]\n",DEFAULT_MAILER);
    fprintf(stderr, "  -M dir   mailbox directory [default=%s]\n",maildir);
    fprintf(stderr, "  -p file  print program with options [default=%s]\n",DEFAULT_PRINTER);
    fprintf(stderr, "  -r       read news remotely from default NNTP server\n",DEFAULT_PRINTER);
    fprintf(stderr, "  -S dir   spool directory [default=%s]\n",SPOOLDIR);
    fprintf(stderr, "  -u       update index files only\n");
    fprintf(stderr, "  -v       update index files only (verbose)\n");
}
